************************************************************
* Download, reshape, annotate and save the public QFR files
* to a stata file
*
* Written to be called from another script, with two arguments:
*	- `1': full path of temporary file where output is saved
*	- `2': full path of permanent file to store the download (if necessary)
*
* Last modified: February 25th, 2015
* Nicolas Crouzet
************************************************************

********************************************************************************************
* The URL for the data is:
* 	http://www.census.gov/econ/qfr/historic.html (rubric: Historical QFR databases)
* Downloadable excel files for 2001-2014 are:
* 	http://www2.census.gov/econ/qfr/hist/QFRHistoricalYYYY.xls,
* where YYYY = year. 
* 
* The code below is based on the assumption that:
* 1) Asset class size encodings don't change (true for 1987-2014)
* 2) Industry classification from latest file can be used to interpret industry classification
* of all other files (true for 2014 back to 2001)
* 3) The reporting formats of each size group don't change
*
* So far, only from 2001-2014; need to extend this to 1987 (industry classifications will
* change + the name of files on the server also changes)
***********************************************************************************************

local TempData = "`1'"
local PermData = "`2'"
local UrlQFR = "http://www2.census.gov/econ/qfr/hist/QFRHistorical"

di "`TempData'"
di "`PermData'"

set obs 1
gen dummy = .
save "`TempData'", replace

/* Download and import data */
forvalues year = 2001(1)2014{

	di in red "Downloading and importing QFR for `year'"
	qui import excel using "`UrlQFR'`year'.xls", describe
	local lastsheet = r(N_worksheet)
	local firstsheet = 2
	* There is an additional sheet with lists of keys for industry groups in 2009
	if `year' == 2009 {
		local ++firstsheet 
	}
	
	forvalues sheet=`firstsheet'/`lastsheet' {

		qui import excel using "`UrlQFR'`year'.xls", describe
		local sheetname=r(worksheet_`sheet')
		qui import excel using "`UrlQFR'`year'.xls", sheet("`sheetname'") clear
		qui sxpose, clear firstnames destring
		qui append using "`TempData'"
		qui save "`TempData'", replace
		
	}

}

/* Drop empty variables and observations */
foreach var of varlist _all {
      capture assert mi(`var')
      if !_rc {
         drop `var'
      }
}
drop if _var1 == ""

/* Create year and quarter identifiers */
gen year = regexs(0) if(regexm(_var1, "[0-9][0-9][0-9][0-9]"))
destring year, replace
lab var year "Year"
format year %ty
gen quarter = regexs(0) if(regexm(_var1, "[0-9]")) 
destring quarter, replace
lab var quarter "Quarter"
gen dateq = yq(year,quarter)
lab var dateq "Quarterly date"
format dateq %tq
drop _var1

/* Label size groups */
rename _var3 assetSize
label define labAssetSize 1 "Total assets under 5 million"
label define labAssetSize 2 "Total assets 5 million to less than $10 million", add
label define labAssetSize 3 "Total assets 10 million to less than $25 million", add
label define labAssetSize 4 "Total assets under 25 million", add
label define labAssetSize 5 "Total assets 25 million and over", add
label define labAssetSize 6 "Total assets 25 million to less than 50 million", add
label define labAssetSize 7 "Total assets 50 million to less than 100 million", add
label define labAssetSize 8 "Total assets 100 million to less than 250 million", add
label define labAssetSize 9 "Total assets 250 million to less than 1 billion", add
label define labAssetSize 10 "Total assets 1 billion and over", add
label define labAssetSize 11 "All total asset sizes (universe totals)", add
label values assetSize labAssetSize
lab var assetSize "Asset size group"

/* Create a variable to track reporting formats */
gen format = ""
replace format = "L" if assetSize <= 4
replace format = "S" if assetSize > 4 & assetSize <= 10
lab var format "Reporting format (see variable labels)"

/* Label industry groups*/
rename _var2 industry
replace industry = "Food" if industry == "311"
replace industry = "Food"	if industry == "311"
replace industry = "Beverage and Tobacco Products"	if industry == "312"
replace industry = "Textile Mills and Textile Product Mills"	if industry == "313"
replace industry = "Apparel and Leather Products"	if industry == "315"
replace industry = "Wood Products"	if industry == "321"
replace industry = "Paper"	if industry == "322"
replace industry = "Printing and Related Support Activities"	if industry == "323"
replace industry = "Petroleum and Coal Products"	if industry == "324"
replace industry = "All Other Chemicals"	if industry == "325"
replace industry = "Plastics and Rubber Products"	if industry == "326"
replace industry = "Nonmetallic Mineral Products"	if industry == "327"
replace industry = "Foundries"	if industry == "331"
replace industry = "Fabricated Metal Products"	if industry == "332"
replace industry = "Machinery"	if industry == "333"
replace industry = "All Other Electronic Products"	if industry == "334"
replace industry = "Electrical Equipment, Appliances, and Components"	if industry == "335"
replace industry = "Furniture and Related Products"	if industry == "337"
replace industry = "Miscellaneous Manufacturing"	if industry == "339"
replace industry = "Iron, Steel, and Ferroalloys"	if industry == "371"
replace industry = "Computer and Peripheral Equipment"	if industry == "374"
replace industry = "Basic Chemicals, Resins, and Synthetics"	if industry == "375"
replace industry = "Motor Vehicles and Parts" 	if industry == "376"
replace industry = "Nonferrous Metals"	if industry == "381"
replace industry = "Communications Equipment"	if industry == "384"
replace industry = "Pharmaceuticals and Medicines"	if industry == "385"
replace industry = "Aerospace Products and Parts"	if industry == "386"
replace industry = "Wholesale Trade, Durable Goods"	if industry == "421"
replace industry = "Wholesale Trade, Nondurable Goods"	if industry == "422"
replace industry = "Food and Beverage Stores"	if industry == "445"
replace industry = "Clothing and General Merchandise Stores"	if industry == "448"
replace industry = "All Other Retail Trade"	if industry == "450"
replace industry = "Publishing Industries, except Internet"	if industry == "511"
replace industry = "Motion Picture and Sound Recording"	if industry == "512"
replace industry = "Broadcasting, except Internet"	if industry == "515"
replace industry = "Telecommunications"	if industry == "517"
replace industry = "All Other Information"	if industry == "519"
replace industry = "Computer Systems Design and Related Services"	if industry == "545"
replace industry = "Management and Technical Consulting Services"	if industry == "546"
replace industry = "Scientific Research and Development Services"	if industry == "547"
replace industry = "All Other Professional and Technical Services (except Legal Services)"	if industry == "549"
replace industry = "All Chemicals" if industry == "CHEM"
replace industry = "All Computer and Electronic Products" if industry == "COMP"
replace industry = "Primary Metals" if industry == "PRIM"
replace industry = "All Durable Manufacturing" if industry == "TDUR"
replace industry = "All Manufacturing" if industry == "TMFG"
replace industry = "All Mining" if industry == "TMIN"
replace industry = "All Nondurable Manufacturing" if industry == "TNDU"
replace industry = "Transportation equipment (residual included in Transportation Equipment" if industry == "TRAN"
replace industry = "All Retail Trade" if industry == "TRET"
replace industry = "All Wholesale Trade" if industry == "TWHS"
replace industry = "All Information" if industry == "TINF"
replace industry = "All Professional and Technical Services (except Legal Services)" if industry == "TPTS"
encode industry, generate(industry_code)
labmask industry_code, values(industry)
drop industry
rename industry_code industry
lab var industry "Industry group"

/* Label industry groups*/
lab var SALES	"Net sales, receipts, and operating revenues"
lab var DEPREC	"Depreciation, depletion, and amortization of property, plant, and equipment"
lab var OTHERCOST	"All other operating costs and expenses"
lab var OPERINC	"Income (loss) from operations"
lab var INTEXP	"Interest expense"
lab var DIVINC	"Dividend income (L only)"
lab var OTHNOPEXP	"Other recurring nonoperating income (expense) (S format) / including interest income, royalties, minority interest, etc. (L format)"
lab var NONRECUR	"Nonrecurring items, including gain (loss) on sale of assets, restructuring costs, asset writedowns, disposal of business segments, etc. (L only)"
lab var EQUITYINC	"Net income (loss) of foreign branches and equity in earnings (losses) of domestic and foreign nonconsolidated subsidiaries and other investments accounted for by the equity method, net of foreign taxes (L only)"
lab var NIBT	"Income (loss) before income taxes"
lab var FEDTAXES	"Provision for current and deferred domestic income taxes (S format) /:  Federal (L format)"
lab var OTHERTAX	"Provision for current and deferred domestic income taxes: Other (L only)"
lab var NIAT	"Income (loss) after income taxes"
lab var EXTRAORD	"Extraordinary gains (losses), net of taxes (L only)"
lab var CUMACTG	"Cumulative effect of accounting changes, net of taxes (L only)"
lab var NIQUARTER	"Net income (loss) for quarter (L only)"
lab var DIVIDENDS	"Cash dividends charged to retained earnings this quarter"
lab var NIRTINBUS	"Net income retained in business"
lab var RETBEG	"Retained earnings at beginning of quarter"
lab var OTHERCRDR	"Other direct credits (charges) to retained earnings (net), including stock and other non-cash dividends, etc."
lab var RETEND	"Retained earnings at end of quarter"
lab var CASH	"Cash and demand deposits in the U.S."
lab var TIMEDEP	"Time deposits in the U.S., including negotiable certificates of deposit"
lab var DEPOUTUS	"Cash and deposits outside the U.S. (L only)"
lab var TOTALCASH	"Total cash on hand and in banks (L format) / in US banks (S format)"
lab var TREASSEC1	"U.S. Treasury and Federal agency securities: Subject to agreements to sell (L only)"
lab var TREASSEC2	"U.S. Treasury and Federal agency securities: Other, due in 1 year or less (L only)"
lab var COMMPAPER	"Commercial and finance company paper of U.S. issuers (L only)"
lab var STATESEC	"State and local government securities, due in 1 year or less (L only)"
lab var FOREGNSEC	"Foreign securities, due in 1 year or less (L only)"
lab var OTHSTINV	"Other short-term financial investments"
lab var TOCASHSEC	"Total cash, U.S. Government and other securities"
lab var RECUSGOVT	"Trade receivables from U.S. Government (L only)"
lab var ACCTRECEV	"Other trade accounts and trade notes receivable (less allowance for doubtful accounts) (L format) / All trade accounts (S format)"
lab var TOTRECEV	"Total receivables (L only)"
lab var INVENTORY	"Inventories"
lab var OTHCURASS	"All other current assets"
lab var TOCRASSET	"Total current assets"
lab var FIXASSETS	"Property, plant, and equipment"
lab var LAND	"Land and mineral rights"
lab var ACCDEPREC	"Accumulated depreciation, depletion, and amortization"
lab var NETPPE	"Net property, plant, and equipment"
lab var TREASSEC3	"U.S. Treasury and Federal agency securities, due in more than 1 year (L only)"
lab var OTHNCASST	"All other noncurrent assets, including investment in nonconsolidated entities, long-term investments, intangibles, etc."
lab var TOTASSET	"Total Assets"
lab var STBANK	"Short-term debt, original maturity 1 year or less: Loans from banks"
lab var COMPAPER	"Short-term debt, original maturity 1 year or less: Commercial paper (L only)"
lab var STDEBTOTH	"Short-term debt, original maturity 1 year or less: Other short-term loans (L format) / including commercial paper (S format)"
lab var ADVUSGOVT	"Advances and prepayments by the U.S. Government (L only)"
lab var ACCTPAY	"Trade accounts and trade notes payable"
lab var FEDTAXPAY	"Income taxes accrued, prior and current years, net of payments: Federal (L only)"
lab var OTHTAXPAY	"Income taxes accrued, prior and current years, net of payments (S format)/ : Other (L format)"
lab var INSTBANKS	"Current portion of long-term debt, due in 1 year or less: Loans from banks"
lab var INSTBONDS	"Current portion of long-term debt, due in 1 year or less:  Bonds and debentures (L only)"
lab var INSTOTHER	"Current portion of long-term debt, due in 1 year or less: Other long-term loans"
lab var OTCURLIAB	"All other current liabilities, including excise and sales taxes, and accrued expenses"
lab var TOCRLIAB	"Total current liabilities"
lab var LTBNKDEBT	"Long-term debt, due in more than 1 year: Loans from banks"
lab var LTBNDDEBT	"Long-term debt, due in more than 1 year: Bonds and debentures (L only)"
lab var LTOTHDEBT	"Long-term debt (due in more than 1 year): Other long-term loans"
lab var OTHNCLIAB	"All other noncurrent liabilities, including deferred income taxes, capitalized leases, and minority stockholders interest in consolidated domestic corporations"
lab var TOTLIAB	"Total liabilities"
lab var CAPSTOCK	"Capital stock and other capital (L format) / less treasury stock (S format)"
lab var RETEARN	"Retained earnings"
lab var CUMFORCUR	"Cumulative foreign currency translation adjustment (L only)"
lab var OTSEITEMS	"Other stockholders equity items, including unearned compensation and ESOP debt guarantees (L only)"
lab var TREASSTK	"Treasury stock at cost (L only)"
lab var STHLDEQUI	"Stockholders equity"
lab var TLIABEQUI	"Total Liabilities and Stockholders Equity"
lab var NETWKCAP	"Net Working Capital: Excess of total current assets over total current liabilities"

/* Describe dataset */
describe

/* Save to temporary file */
save "`TempData'", replace

/* Save to permanent file if a permanent file path was passed as argument */
if strlen("`PermData'") > 1 {
	save "`PermData'", replace
}
