% List of figures constructed by this script:
%
% Section 3
%
%   % figure  5a - optimal composition of debt
%   % figure  5b - borrowing levels
%   % figure  6  - cross-sectional distribution
%   % figure  7  - other aspect of firm dynamics
%   % figure  8  - optimal policies with interim equity issuance
%
% Section 4
%
%   % figure  9  - patterns of debt composition in US data during the Great Recession
%   % figure  10 - response to a credit supply shock
%   
% Section 5 
%
%   % figure  12 - effects of subsidies to bond issuance costs for small firms

% Initialization
clear all;
close all;
warning('off','MATLAB:mir_warning_maybe_uninitialized_temporary')
rng(1);
clc;
poolobj = gcp('nocreate');
if isempty(poolobj)
    c = parcluster('local');  % build the 'local' cluster object
    nw = c.NumWorkers;        % get the number of workers
    parpool('local',nw);
end;
clear poolobj;

% Set filepaths
home_dir = pwd;
local_dir = pwd;
addpath([local_dir '/xxx - auxiliary functions/computation'])
addpath([local_dir '/xxx - auxiliary functions/graphing'])
addpath([local_dir ''])
cd(home_dir);
savepath = [local_dir '/figures/'];
modelsolutionspath = [local_dir '/model solutions/'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve versions of the model necessary to construct the graphs
% Note: solution calls have been commented out to save time for 
% replication; uncomment them in order to run the solution again
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Paralellized = true: paralellized by z within each model
% Paralellized = false: paralellized by model
parameters_calibration.paralellized_inside  = true;

% Fixed parameters (others are set inside of solve_steady_state_functional)
parameters_calibration.chi_P                = 0.675;
parameters_calibration.zeta                 = 0.975;
parameters_calibration.delta_k              = 0.2;
parameters_calibration.N                    = 3e2+1;
parameters_calibration.H                    = 15;

% Change to verify a particular calibration
parameters_calibration.optimize_e           = true; 
parameters_calibration.gamma_e_i            = NaN;
parameters_calibration.iter_e               = 20;
parameters_calibration.print_stuff          = false;
parameters_calibration.interim_equity       = false;
parameters_calibration.no_exit_c            = false;
parameters_calibration.subsidy              = false;
parameters_calibration.e_subsidy            = NaN;

% Initial steady-state: 2007Q3
parameters_calibration.r_b_vec = 0.00735;
    
    % Uncomment to re-calibrate the model
    %[model,output] = calibration(parameters_calibration);
    %save([modelsolutionspath 'steady_state_2007Q3'],'model','output');

load([modelsolutionspath 'steady_state_2007Q3']);
N_models = size(output,2);
BS       = NaN(N_models,1);
for ii=1:N_models
    BS(ii) = output(ii).BS;
end;
index_model_2007Q3 = find(min(abs(BS-0.24)) == abs(BS-0.24));
model_2007Q3 = model(index_model_2007Q3);
output_2007Q3 = output(index_model_2007Q3);

% Allow for interim equity issuance
load([modelsolutionspath 'steady_state_2007Q3']);
parameters_calibration.r_b_vec          = model(index_model_2007Q3).rb;
parameters_calibration.interim_equity   = true;
parameters_calibration.optimize_e       = false; 
parameters_calibration.gamma_e_i        = output(index_model_2007Q3).gamma_e_ipo;
    
    % Uncomment to re-calibrate the model
    %[model,output] = calibration(parameters_calibration);
    %save([modelsolutionspath 'steady_state_2007Q3_interim_equity'],'model','output');

% Long-run steady-state: 2011Q3
load([modelsolutionspath 'steady_state_2007Q3']);
parameters_calibration.r_b_vec          = 0.00810;
parameters_calibration.interim_equity   = false;
parameters_calibration.optimize_e       = false;
parameters_calibration.subsidy          = false;
parameters_calibration.no_exit_c        = false;
parameters_calibration.gamma_e_i        = output(index_model_2007Q3).gamma_e_ipo;

    % Uncomment to re-calibrate the model
    %[model,output] = calibration(parameters_calibration);
    %save([modelsolutionspath 'steady_state_2011Q3'],'model','output');

load([modelsolutionspath 'steady_state_2011Q3']);
N_models = size(output,2);
BS       = NaN(N_models,1);
for ii=1:N_models
    BS(ii) = output(ii).BS;
end;
index_model_2011Q3 = find(min(abs(BS-0.202)) == abs(BS-0.202));
model_2011Q3 = model(index_model_2011Q3);
output_2011Q3 = output(index_model_2011Q3);

% Solve for the transition path
clear model output
output(1)= output_2007Q3;
output(2) = output_2011Q3;

    % Uncomment to re-solve the transition path
    %results_transition_path = transition_path(output);
    %save([modelsolutionspath 'transition_path'],'results_transition_path');
    
% Uncomment to re-construct the equilibrium with subsidies
%{

load([modelsolutionspath 'steady_state_2007Q3']);
parameters_calibration.r_b_vec          = model(index_model_2007Q3).rb;
parameters_calibration.interim_equity   = false;
parameters_calibration.optimize_e       = false; 
parameters_calibration.subsidy          = true;
parameters_calibration.e_subsidy        = output(index_model_2007Q3).policy_functions.e_switch_nosubsidy(3);
parameters_calibration.print_stuff      = true;
    
    [model,output]                          = calibration(parameters_calibration);
    save([modelsolutionspath 'steady_state_2007Q3_subsidy'],'model','output');   

%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct the non-tikz graphs from the paper
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

% Section 3 graphs

    load([modelsolutionspath 'steady_state_2007Q3.mat']);
    policy_functions                = output.policy_functions;
    policy_functions_baseline       = policy_functions;
    distribution                    = output.distribution;
    pars                            = output.pars;

    % Figure 5a
    figname = 'fig5a'; fignum = 1; has_arrows = true;
    fig5a(policy_functions,figname,savepath,fignum,has_arrows);

    % Figure 5b
    figname = 'fig5b'; fignum = 2; has_arrows = true;
    fig5b(policy_functions,figname,savepath,fignum,has_arrows);
    
    % Figure 6
    figname = 'fig6'; fignum = 3; has_arrows = true;
    fig6(policy_functions,distribution,figname,savepath,fignum,has_arrows);
    
    % Figure 7
    figname = 'fig7'; fignum = 4; has_arrows = true;
    fig7(policy_functions,pars,figname,savepath,fignum,has_arrows);
    
    % Figure 8
    load([modelsolutionspath 'steady_state_2007Q3_interim_equity']);
    policy_functions = output.policy_functions;
    figname = 'fig8'; fignum = 5; has_arrows = true;
    fig8(policy_functions,policy_functions_baseline,pars,figname,savepath,fignum,has_arrows);

% Section 4 graphs

    % Figure 9
    figname = 'fig9'; fignum = 6;
    fig9(savepath,figname,fignum);

    % Figure 10
    load([modelsolutionspath 'transition_path.mat']);
    figname = 'fig10'; fignum = 7; 
    fig10(results_transition_path,figname,savepath,fignum) 

% Section 5 graphs

    % Figure 12
    load([modelsolutionspath 'steady_state_2007Q3']);
    output_nosubsidy = output;
    load([modelsolutionspath 'steady_state_2007Q3_subsidy']);
    output_subsidy = output;
    figname = 'fig12'; fignum = 8; has_arrows = false;
    fig12(output_nosubsidy,output_subsidy,figname,savepath,fignum,has_arrows);
