function [x,fx,counter] = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning)

% This function implements a simple bissection algorithm on [x_low,x_high].
% The continuous function for which a zero is computed is called by the handle ff.
% The handle ff takes two arguments: x, and indexes, a vector of indexes
% which must be the same size as x.
%
% The function MUST be strictly increasing, its value at the left edge must be <=0, and its value at the right edge must be >=0. 
% The function can take infinite values at the edges.
%
% The function checks for the signs at the boundaries, but not the monotonicity, so the user has to make sure that the function
% is monotonic before the call.
%
% Inputs:
%  
%  - x_low          size numargs*1               values in [0,+\infty[
%  - x_high         size numargs*1               values in [0,+\infty[
%  - ff             function handle              values in [0,+\infty]
%  - tol_f          tolerance for the objective 
%  - tol_x          tolerance for the arguments
%
% Outputs:
%
%  - x          size numargs*1               values in [0,+\infty[      (solution to ff(x) = 0)
%  - fx         size numargs*1               values in [0,+\infty[      (value of ff(x), abs(ff(x)) tol_f)
%
% This version assumes that the solution is not at one of the boundaries.
%
% This function is linearized, i.e. it solves f(x) = 0_{n\times1} when f is
% vector-valued.
%
% Nicolas Crouzet, February 2016

x = NaN(size(x_low)); fx = x; numargs = numel(x_low); indexes = (1:numargs)';

% Values at initial bounds
fx_low      = ff(x_low,indexes);
fx_high     = ff(x_high,indexes);

% Three cases
case1 = ( fx_low < -tol_f | fx_high > tol_f | abs(x_low - x_high) >  tol_x );

if sum(case1) ~= numargs
    error('Wrong cases in bissection_noedges \n');
end; 

if sum( case1&( fx_low > 0 | fx_high < 0 | (x_high < x_low) | isinf(x_high) | isinf(x_low)) ) > 0
    error('Sign conditions at the edges violated in bissection_noedges \n');
end;

if sum(case1) > 0

    x_mid               = NaN(numargs,1);
    fx_mid              = 10*ones(numargs,1); 
    counter             = 1;
    has_not_converged   = case1;
    
    while ( sum( has_not_converged ) > 0 )&&( counter < 2e2 )
        
        x_mid(case1)      = x_low(case1)+(1/2)*(x_high(case1) - x_low(case1));
        fx_mid(case1)     = ff(x_mid(case1),indexes(case1));

        % This assumes that fx_low <= fx_mid <= fx_high always holds
        update_low        = (case1)&( fx_mid <= 0 );
        update_high       = (case1)&( fx_mid > 0 );
           
        x_low(case1&update_low)      = x_mid(case1&update_low);
        fx_low(case1&update_low)     = fx_mid(case1&update_low);
        x_high(case1&update_high)    = x_mid(case1&update_high);
        fx_high(case1&update_high)   = fx_mid(case1&update_high);
        
        counter             = counter + 1;
        has_not_converged   = case1&( fx_low < -tol_f | fx_high >  tol_f | ( abs(x_low - x_mid) >  tol_x & abs(x_high - x_mid) >  tol_x ) );
        
        %if strcmp(return_warning{2},'switching threshold')
        %    [x_low fx_low]
        %    [x_high fx_high]
        %end;
        
    end;
    
    if ( sum( has_not_converged  ) > 0 && return_warning{1} )
        fprintf('     Warning: a call to the bissection algorithm did not converge for %d out of %d elements \n',sum(has_not_converged),numargs); 
        fprintf(['     The call came from ' return_warning{2} ' \n']);
        fprintf('          x_low  = %3.25f \n',x_low( has_not_converged ));
        fprintf('          x_high = %3.25f \n',x_high( has_not_converged ));
        fprintf('          x_high - x_low = %3.25f \n',x_high( has_not_converged ) - x_low( has_not_converged ));
        fprintf('          tol            = %3.25f \n',tol_x);
        fprintf('          fx_low  = %3.25f \n',fx_low( has_not_converged ));
        fprintf('          fx_high = %3.25f \n',fx_high( has_not_converged ));
        fprintf('          fx_high - fx_low = %3.25f \n',fx_high( has_not_converged ) - fx_low( has_not_converged ));
        fprintf('          tol              = %3.25f \n',tol_f);
        fprintf('\n');
    end;
        
    x(case1)    = x_high(case1); 
    fx(case1)   = fx_high(case1);
    
end;

end

