function [rt,lb,ub] = find_root_fractional(A,B,tol_x,tol_f,pars)

% This function computes the largest strictly positive solution to the equation:
%           Ak^(zeta) = k - B
% where A > 0, zeta > 1/2, B >= 0.
%
% It uses the bissection algorithm as well as simpler bounds on the root
%
% Nicolas Crouzet, March 2017.

cases_not_treated = (A <= 0 | B < 0 | pars.zeta <= 1/2);
if sum(cases_not_treated) > 0
    error('find_root_fractional received non-treated cases \n');
end;

numargs = numel(A);
if numargs ~=  numel(B)
    error('wrong size of arguments in find_root_fractional \n');
end;
rt = NaN(numargs,1); lb = rt; ub = rt;

glb = zeros(numargs,1);
gub = max(A.^(-1/pars.zeta)-1,B);
klb = A.^(1/(1-pars.zeta));
kub = (1+gub).^(1/(1-pars.zeta)).*klb;

case1 = (B == 0);
case2 = (B > 0);

if sum(case1) > 0
    
    rt(case1,1) = A(case1).^(1/(1-pars.zeta));
    lb(case1,1) = rt(case1,1);
    ub(case1,1) = rt(case1,1);
    
end;

if sum(case2) > 0
    
        A2 = A(case2); B2 = B(case2);

        % Compute the root
        x_low = glb(case2); x_high = gub(case2);
        ff  = @(x,indexes) ((x).^(1-pars.zeta)).*((1+x).^(pars.zeta)) - (B2(indexes).^(1-pars.zeta)).*(A2(indexes).^(-1));
        return_warning{1} = true; return_warning{2} = 'find_root_fractional';
        g2 = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
        
        % Store the results
        rt(case2) = ((1+g2).^(1/(1-pars.zeta))).*klb(case2);
        lb(case2) = klb(case2);
        ub(case2) = kub(case2);
        
end;

end


