function x = intspace( x_low, x_high, n, xi )

% intspace.m
%
% This function creates a vector of n points spaced in an ``intermediate"
% manner, relative to two extreme cases: linear spacing and log spacing.
%
% Linear spacing is:
%
%    x_{i+1} = x_{i} + Delta
% 
% where Delta = 1/(n-1)*(x_high - x_low), whereas log-spacing is:
%
%    log(x_{i+1}) = log(x_{i}) + Delta
%
% where Delta = 1/(n-1)*log(x_low/x_high). 
%
% "Intermediate" spacing is:
%
%    x_{i+1}^{xi} = x_i^{xi} + Delta
%
% where Delta = 1/(n-1)*(x_high^(xi) - x_low^(xi)). Here, xi is a parameter
% between 0 and 1, which controls how close one is from the linear or the
% log case. As xi -> 1, one converges to the linear case (trivially) and as
% xi -> 0, one converges to the log case (haven't proved it formally).
% Thus, for xi - > 0, one obtains a vector with points concentrated
% towards x_low (which converge to an arithmetic sequence), while for xi -> 1, one obtains 
% a regularly spaced vector (forming an arithmetic sequence). The advantage
% relative to log-spacing is that this works even when x_low = 0.
%
% Nicolas Crouzet, October 2015

Delta       = 1/(n-1)*( x_high^xi - x_low^xi );
logy        = x_low^xi + Delta*( cumsum( ones(n,1) ) - 1 );
x           = ( logy ).^(1/xi);

end

