function results = solve_steady_state_functional(model)

if model.print_stuff

            fprintf('\n');
            fprintf('------------------------------------------------------  \n');
            fprintf('---- Solution for chi = %7.6f, r_b = %7.6f -----  \n',model.chi,model.rb);        
            fprintf('------------------------------------------------------  \n');
            
end;            

if model.model_version==1 && model.print_stuff
    
    fprintf(' \n');
    fprintf('Dynamic model with two debt instruments \n');
    fprintf(' \n');
    
end;

if model.model_version==2 && model.print_stuff
    
    fprintf(' \n');
    fprintf('Dynamic model with one debt instrument \n');
    fprintf(' \n');
    
end;

if model.model_version==3 && model.print_stuff
    
    fprintf(' \n');
    fprintf('Static model with two debt instruments \n');
    fprintf(' \n');
    
end;

if model.model_version==4 && model.print_stuff
    
    fprintf(' \n');
    fprintf('Static model with one debt instrument \n');
    fprintf(' \n');
    
end;

% Main code piece
if model.solve_everything 

            % Create optimization parameters
            pars = create_optimization_parameters;   
            
            % Used paralellization by value of z or not
            pars.paralellized_inside = model.paralellized_inside;
            
            % Record whether one or two debt instruments
            pars.onei = false;
            if model.model_version == 2 || model.model_version == 4
                pars.onei = true;
            end;

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Parameters not related to the productivity 

            pars.beta             = 1/1.02;                         % Discount rate
            pars.r                = 1/pars.beta - 1;                % Risk-free rate
            pars.zeta             = model.zeta;                     % Degree of decreasing returns to scale
            pars.delta_k          = model.delta_k;                  % Rate of depreciation
            pars.chi              = 0.38;                          
            pars.gamma_m          = 0.0100;                         % Market intermediation costs
            pars.r_m              = pars.r + pars.gamma_m;          % Total market lending intermediation costs
            pars.gamma_b          = pars.gamma_m + model.rb;        % Bank intermediation costs
            pars.r_b              = pars.r + pars.gamma_b;          % Total bank lending costs
            pars.gamma_e          = 10.0;                           % Cost of equity issuance (set to > 5 or so to get no equity issuance)
            if model.interim_equity
                pars.gamma_e = 0.1;
            end;
            pars.eta              = 0.05;                           % Exogenous exit rate
            pars.xi               = 0.5;                            % Fraction consumed upon exit
            if model.no_exit_c
                pars.xi           = 0;
            end;
            pars.N_e              = 1;                              % Number of potential entrants
            pars.lambda           = 5;                              % Coefficient on exponential distribution for fixed entry cost
            pars.chi_L            = model.chi;                      % Parameter governing the resale value of depreciated capital outside of bankruptcy. 
                                                                    % =1 is the baseline (depreciated capital is fungible with output).
                                                                    % Cannot be smaller than pars.chi                                         

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Parameters controlling tax rates and thresholds for 
            % subsidies -- set a negative value for pars.e_subsidy for these
            % to be ignored

            if ~model.subsidy
            
                pars.e_subsidy          = -10;

                pars.tau                = 0.0;
                pars.tau_m_nosubsidy    = 0.0;
                pars.tau_m_subsidy      = 0.0;

                pars.r_m_nosubsidy      = pars.r_m;
                pars.r_m_nosubsidy_R    = (1-pars.tau_m_nosubsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.r_m_nosubsidy_NR   = (1-pars.tau_m_nosubsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.chi_m_nosubsidy    = pars.chi*(1-pars.tau_m_nosubsidy)/(1-pars.tau);

                pars.r_m_subsidy        = pars.r_m;
                pars.r_m_subsidy_R      = (1-pars.tau_m_subsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.r_m_subsidy_NR     = (1-pars.tau_m_subsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.chi_m_subsidy      = pars.chi*(1-pars.tau_m_subsidy)/(1-pars.tau);
                
            else
                
                pars.e_subsidy          = model.e_subsidy;

                pars.tau                = 0.0;
                pars.tau_m_nosubsidy    = 0.0;
                pars.tau_m_subsidy      = 0.0;

                pars.r_m_nosubsidy      = pars.r_m;
                pars.r_m_nosubsidy_R    = (1-pars.tau_m_nosubsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.r_m_nosubsidy_NR   = (1-pars.tau_m_nosubsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.chi_m_nosubsidy    = pars.chi*(1-pars.tau_m_nosubsidy)/(1-pars.tau);

                pars.r_m_subsidy        = pars.r_m-0.0020;
                pars.r_m_subsidy_R      = (1-pars.tau_m_subsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.r_m_subsidy_NR     = (1-pars.tau_m_subsidy)/(1-pars.tau)*(1+pars.r_m) - 1;
                pars.chi_m_subsidy      = pars.chi*(1-pars.tau_m_subsidy)/(1-pars.tau);
                
            end;

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Calibration of the productivity process

            pars.N_z              = 3;                              % Number of exogenous productivity states

            if pars.N_z == 1

                % Normalization for maximum size of largest firm
                pars.k_star         = NaN(pars.N_z,1); 
                pars.k_star(end)    = 100;
                pars.e_star         = NaN(pars.N_z,1);
                pars.e_star(end)    = (pars.r_m+pars.delta_k)/(1+pars.r_m)*pars.k_star(end);
                pars.E_phi          = NaN(pars.N_z,1); 
                pars.E_phi(end)     = (pars.delta_k+pars.r_m)/pars.zeta*(pars.k_star(end))^(1-pars.zeta);

                % Cross-sectional variance
                var_logz = 0.40654^2;
                pars.b   = sqrt(pi^2/(6*var_logz));

                % Other parameters
                pars.var_phi = NaN(pars.N_z,1); pars.a = NaN(pars.N_z,1); pars.FF = cell(pars.N_z,1); pars.k_star_B = NaN(pars.N_z,1);
                pars.a(end)           = pars.E_phi(end)/gamma(1+1/pars.b(end));
                pars.FF{end}          = makedist('weibull','a',pars.a(end),'b',pars.b(end));
                pars.var_phi(end)     = std(pars.FF{end})^2;
                pars.k_star_B(end)    = (pars.zeta*pars.E_phi(end)/(pars.r+pars.delta_k)).^(1/(1-pars.zeta));

                % Transition matrix
                pars.ppi              = 1;
                pars.pi_ergodic       = 1;

            else

                % Target autocorrelation for log-productivity
                pars.rho_logz = 0.7733;

                % Target relative optimal size of implied by smallest vs. largest
                % productivity level in state-space
                pars.rel_size = 0.0831;

                % Construct the implied discrete-state Markov chain for
                % log-productivity
                psi = -(1-pars.zeta)/2*log(pars.rel_size);
                p = (1+pars.rho_logz)/2; q = p;
                [ logz, ppi, pi_ergodic, moments ] = rouwenhorst_approximation(pars.N_z,psi,p,q);

                % Transition matrix and ergodic distribution
                pars.ppi = ppi;
                pars.pi_ergodic = pi_ergodic; 

                % Normalization for maximum size of largest firm
                pars.k_star         = NaN(pars.N_z,1); 
                pars.k_star(end)    = 100;
                pars.e_star         = NaN(pars.N_z,1);
                pars.e_star(end)    = (pars.r_m+pars.delta_k)/(1+pars.r_m)*pars.k_star(end);
                pars.E_phi          = NaN(pars.N_z,1); pars.z_av = NaN(pars.N_z,1);
                pars.E_phi(end)     = (pars.delta_k+pars.r_m)/pars.zeta*(pars.k_star(end))^(1-pars.zeta);
                pars.z_av(end)      = pars.E_phi(end);

                % State-space for average productivity z
                pars.mu_z           = log(pars.z_av(end)) + (1-pars.zeta)/2*log(pars.rel_size);
                pars.z_av           = exp(logz+pars.mu_z);

                % Characteristics of the conditional distribution of phi_t for each
                % state z
                pars.var_phi = NaN(pars.N_z,1); pars.a = NaN(pars.N_z,1); pars.b = NaN(pars.N_z,1); pars.z_lr = NaN(pars.N_z,1); pars.FF = cell(pars.N_z,1);
                pars.k_star_B = NaN(pars.N_z,1);

                % Match the total unconditional variance of log(phi_t)
                var_logz = 0.40654^2;
                var_log_epsilon = var_logz - moments.uncond_var;
                pars.b(:) = sqrt(pi^2/(6*var_log_epsilon));

                % Complete the vector 
                for m = 1:pars.N_z
                                pars.E_phi(m)       = pars.z_av(m);
                                pars.a(m)           = pars.E_phi(m)/gamma(1+1/pars.b(m));
                                pars.FF{m}          = makedist('weibull','a',pars.a(m),'b',pars.b(m));
                                pars.var_phi(m)     = std(pars.FF{m})^2;
                                pars.k_star(m)      = (pars.zeta*pars.E_phi(m)/(pars.r_m+pars.delta_k)).^(1/(1-pars.zeta));
                                pars.e_star(m)      = (pars.r_m+pars.delta_k)/(1+pars.r_m)*pars.k_star(m);          
                                pars.k_star_B(m)    = (pars.zeta*pars.E_phi(m)/(pars.r+pars.delta_k)).^(1/(1-pars.zeta));
                end;

            end;

            pars.z                          = [pars.a pars.b pars.ppi];
            pars.z_baseline                 = pars.z;
            pars.z_augmented_nosubsidy      = [pars.z repmat(pars.chi_m_nosubsidy,[pars.N_z,1]) repmat(pars.r_m_nosubsidy_NR,[pars.N_z,1]) repmat(pars.r_m_nosubsidy_R,[pars.N_z,1])];
            pars.z_augmented_subsidy        = [pars.z repmat(pars.chi_m_subsidy,[pars.N_z,1]) repmat(pars.r_m_subsidy_NR,[pars.N_z,1]) repmat(pars.r_m_subsidy_R,[pars.N_z,1])];

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Other parameters

            % Size of state vector
            % This is the number of elements in the A and B vectors that characterize the continuation value
            % ecv has N+1 elements per productivity level, and e has N-1 elements per productivity level.
            pars.N = model.N;

            % Initialization of matrices characterizing continuation values
            % This is equivalent to solving for the limit of the finite-horizon economy
            A = zeros(pars.N_z,pars.N); B = zeros(pars.N_z,pars.N); ecv = zeros(pars.N_z,pars.N+1);

            % Number of times to iterate the contraction mapping
            Horizon = model.H; counter = 0; 
            if model.model_version == 3 || model.model_version == 4
                Horizon = 0;
            end;

            % For computation of firm dynamics moments
            pars.threshold_SL     = 2.5;
            pars.FD_index         = pars.N_z;

            % Print model calibration
            % print_model_calibration(pars);

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
            
            %%%%%%%%%%%%%%%%%
            % Model solution
            %%%%%%%%%%%%%%%%%

            % Uncomment to get the model's solution

            tt_total = tic;

            % Policy functions
            while counter <= Horizon

                tt = tic;
                if (mod(counter,1))==0 && model.print_stuff
                    fprintf('Horizon %d, counter %d \n',Horizon-counter, counter);
                end;

                if counter > 0
                    V_previous = policy_functions.V; e_previous = policy_functions.e; 
                    e_switch_previous = policy_functions.e_switch_nosubsidy; e_div_previous = policy_functions.e(end,:);
                end;

                if model.paralellized_inside
                    [ ecvp, Ap, Bp, policy_functions ] = iterate_model_paralellized( ecv, A, B, pars, [], ((mod(counter,1))==0)&&(model.print_stuff) ); 
                else
                    [ ecvp, Ap, Bp, policy_functions ] = iterate_model( ecv, A, B, pars, [], ((mod(counter,1))==0)&&(model.print_stuff) ); 
                end;
                A = Ap; B = Bp; ecv = ecvp; counter = counter+1; 
  
                tt = toc(tt);
                if (mod(counter,1))==0 && model.print_stuff
                    fprintf('Time for horizon %d: %d min and %d seconds \n',counter,floor(tt/60),round(mod(tt,60))); fprintf('\n');
                end;    

            end;
            tt_total = toc(tt_total);
            if model.print_stuff
                fprintf('Overall time for iterations: %d min and %d seconds \n',floor(tt_total/60),round(mod(tt_total,60)));
                fprintf('\n')
            end;

            % Final converge criteria for policy functions
            e_div    = policy_functions.e(end,:); 
            e_switch = policy_functions.e_switch_nosubsidy; 
            V        = policy_functions.V;
            
            if ( model.model_version == 1 || model.model_version == 2 )&&( counter > 1 )
            
                for m = 1:pars.N_z
                    
                    if model.print_stuff
                    
                        fprintf('\n');
                        fprintf('Convergence criteria for exogenous state %d \n',m);
                        fprintf('-------------------------------------------\n');
                        fprintf('\n');

                        fprintf('Net worth thresholds: \n');
                        fprintf('   e_switch  = %7.3f, e_switch previous  = %7.3f, final change = %7.3f \n',e_switch(m),e_switch_previous(m),e_switch(m) - e_switch_previous(m));
                        fprintf('   e_div     = %7.3f, e_div previous     = %7.3f, final change = %7.3f \n',e_div(m),e_div_previous(m),e_div(m) - e_div_previous(m));
                        fprintf('\n');

                        dV(m) = max(abs(V(:,m) - V_previous(:,m)));
                        fprintf('Change in value function \n');
                        fprintf('   max V change = %6.3f \n',dV(m)); 
                        fprintf('\n');
                        
                    end;
                    
                end;
                
            end;
      
else            

            load(model.file_solution, '-regexp', '^(?!model(ii))\w')
    
end;

% Minimize distance with target moments
steady_state = true; current_mu = []; next_e = [];
if model.model_version == 3 || model.model_version == 4
      A = 0; B = 0;
end;
gamma_e_ipo = model.gamma_e_i;
if model.optimize_e
    ff = @(x)  matching_target(x,policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars,false);
    x_min = 0.05; x_max = 1.45;
    options = optimset('Display','off','MaxIter',model.maxiter_e);
    gamma_e_ipo = fminbnd(ff,x_min,x_max,options);
end;

% Compute features of the stationary distribution
if pars.paralellized_inside
    [BS,Mixed_Frac,ag_lev,nw_entry_ratio,~,distribution,aggregates] = aggregate_implications_for_moment_matching_paralellized(gamma_e_ipo,policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars,false);
else
    [BS,Mixed_Frac,ag_lev,nw_entry_ratio,~,distribution,aggregates] = aggregate_implications_for_moment_matching(gamma_e_ipo,policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars,false);
end;

% Store results
results.pars                = pars;
results.ecv                 = ecv;
results.A                   = A;
results.B                   = B;
results.policy_functions    = policy_functions;
results.distribution        = distribution;
results.aggregates          = aggregates;
results.gamma_e_ipo         = gamma_e_ipo;
results.BS                  = BS;
results.mixed_frac          = Mixed_Frac;
results.ag_lev              = ag_lev;
results.nw_entry_ratio      = nw_entry_ratio;
results.assets_drop         = aggregates.assets_drop;

end


    
